@@ -1737,57 +1737,7 @@ gsasl_md5pwd_get_password (const char *filename,
   return GSASL_AUTHENTICATION_ERROR;
 }
 
-
-/*
- * Copyright (c) 1996-1999 by Internet Software Consortium.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM DISCLAIMS
- * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE
- * CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
- * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
- * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
- * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- */
-
-/*
- * Portions Copyright (c) 1995 by International Business Machines, Inc.
- *
- * International Business Machines, Inc. (hereinafter called IBM) grants
- * permission under its copyrights to use, copy, modify, and distribute this
- * Software with or without fee, provided that the above copyright notice and
- * all paragraphs of this notice appear in all copies, and that the name of IBM
- * not be used in connection with the marketing of any product incorporating
- * the Software or modifications thereof, without specific, written prior
- * permission.
- *
- * To the extent it has a right to do so, IBM grants an immunity from suit
- * under its patents, if any, for the use, sale or manufacture of products to
- * the extent that such products are used for performing Domain Name System
- * dynamic updates in TCP/IP networks by means of the Software.  No immunity is
- * granted for any product per se or for any other function of any product.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", AND IBM DISCLAIMS ALL WARRANTIES,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
- * PARTICULAR PURPOSE.  IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL,
- * DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER ARISING
- * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE, EVEN
- * IF IBM IS APPRISED OF THE POSSIBILITY OF SUCH DAMAGES.
- */
-
-#include <ctype.h>
-#include <stdio.h>
-
-#define Assert(Cond) if (!(Cond)) abort()
-
-static const char Base64[] =
-  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-static const char Pad64 = '=';
+#include <minmax.h>
 
 /**
  * gsasl_base64_encode:
@@ -1809,64 +1759,20 @@ int
 gsasl_base64_encode (char const *src,
 		     size_t srclength, char *target, size_t targsize)
 {
-  size_t datalength = 0;
-  unsigned char input[3];
-  unsigned char output[4];
-  size_t i;
+  int rc;
+  char *out;
+  size_t outlen;
+  int copied;
 
-  while (2 < srclength)
-    {
-      input[0] = *src++;
-      input[1] = *src++;
-      input[2] = *src++;
-      srclength -= 3;
+  rc = gsasl_base64_to (src, srclength, &out, &outlen);
+  if (rc)
+    return -1;
 
-      output[0] = input[0] >> 2;
-      output[1] = ((input[0] & 0x03) << 4) + (input[1] >> 4);
-      output[2] = ((input[1] & 0x0f) << 2) + (input[2] >> 6);
-      output[3] = input[2] & 0x3f;
-      Assert (output[0] < 64);
-      Assert (output[1] < 64);
-      Assert (output[2] < 64);
-      Assert (output[3] < 64);
+  copied = MIN (outlen, targsize);
+  memcpy (target, out, copied);
+  free (out);
 
-      if (datalength + 4 > targsize)
-	return (-1);
-      target[datalength++] = Base64[output[0]];
-      target[datalength++] = Base64[output[1]];
-      target[datalength++] = Base64[output[2]];
-      target[datalength++] = Base64[output[3]];
-    }
-
-  /* Now we worry about padding. */
-  if (0 != srclength)
-    {
-      /* Get what's left. */
-      input[0] = input[1] = input[2] = '\0';
-      for (i = 0; i < srclength; i++)
-	input[i] = *src++;
-
-      output[0] = input[0] >> 2;
-      output[1] = ((input[0] & 0x03) << 4) + (input[1] >> 4);
-      output[2] = ((input[1] & 0x0f) << 2) + (input[2] >> 6);
-      Assert (output[0] < 64);
-      Assert (output[1] < 64);
-      Assert (output[2] < 64);
-
-      if (datalength + 4 > targsize)
-	return (-1);
-      target[datalength++] = Base64[output[0]];
-      target[datalength++] = Base64[output[1]];
-      if (srclength == 1)
-	target[datalength++] = Pad64;
-      else
-	target[datalength++] = Base64[output[2]];
-      target[datalength++] = Pad64;
-    }
-  if (datalength >= targsize)
-    return (-1);
-  target[datalength] = '\0';	/* Returned value doesn't count \0. */
-  return (datalength);
+  return copied;
 }
 
 /**
@@ -1887,126 +1793,261 @@ gsasl_base64_encode (char const *src,
 int
 gsasl_base64_decode (char const *src, char *target, size_t targsize)
 {
-  int tarindex, state, ch;
-  char *pos;
+  int rc;
+  char *out;
+  size_t outlen;
+  int copied;
 
-  state = 0;
-  tarindex = 0;
+  rc = gsasl_base64_from (src, strlen (src), &out, &outlen);
+  if (rc)
+    return -1;
 
-  while ((ch = *src++) != '\0')
-    {
-      if (isspace (ch))		/* Skip whitespace anywhere. */
-	continue;
+  copied = MIN (outlen, targsize);
+  memcpy (target, out, copied);
+  free (out);
 
-      if (ch == Pad64)
+  return copied;
+}
+
+const char *
+_gsasl_obsolete_property_map (Gsasl_session * sctx, Gsasl_property prop)
+{
+  char buf[BUFSIZ];
+  size_t buflen = BUFSIZ - 1;
+  int res;
+
+  buf[0] = '\0';
+
+  /* Translate obsolete callbacks to modern properties. */
+
+  switch (prop)
+    {
+    case GSASL_SERVICE:
+      {
+	Gsasl_client_callback_service cb_service
+	  = gsasl_client_callback_service_get (sctx->ctx);
+	if (!cb_service)
+	  break;
+	res = cb_service (sctx, buf, &buflen, NULL, 0, NULL, 0);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    case GSASL_HOSTNAME:
+      {
+	Gsasl_client_callback_service cb_service
+	  = gsasl_client_callback_service_get (sctx->ctx);
+	if (!cb_service)
+	  break;
+	res = cb_service (sctx, NULL, 0, buf, &buflen, NULL, 0);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    case GSASL_ANONYMOUS_TOKEN:
+      {
+	Gsasl_client_callback_anonymous cb_anonymous
+	  = gsasl_client_callback_anonymous_get (sctx->ctx);
+	if (!cb_anonymous)
+	  break;
+	res = cb_anonymous (sctx, buf, &buflen);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    case GSASL_AUTHID:
+      {
+	Gsasl_client_callback_authentication_id cb_authentication_id
+	  = gsasl_client_callback_authentication_id_get (sctx->ctx);
+	if (!cb_authentication_id)
+	  break;
+	res = cb_authentication_id (sctx, buf, &buflen);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    case GSASL_AUTHZID:
+      {
+	Gsasl_client_callback_authorization_id cb_authorization_id
+	  = gsasl_client_callback_authorization_id_get (sctx->ctx);
+	if (!cb_authorization_id)
+	  break;
+	res = cb_authorization_id (sctx, buf, &buflen);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    case GSASL_PASSWORD:
+      {
+	Gsasl_client_callback_password cb_password
+	  = gsasl_client_callback_password_get (sctx->ctx);
+	if (!cb_password)
+	  break;
+	res = cb_password (sctx, buf, &buflen);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    case GSASL_PASSCODE:
+      {
+	Gsasl_client_callback_passcode cb_passcode
+	  = gsasl_client_callback_passcode_get (sctx->ctx);
+	if (!cb_passcode)
+	  break;
+	res = cb_passcode (sctx, buf, &buflen);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
 	break;
+      }
 
-      pos = strchr (Base64, ch);
-      if (pos == 0)		/* A non-base64 character. */
-	return (-1);
-
-      switch (state)
-	{
-	case 0:
-	  if (target)
-	    {
-	      if ((size_t) tarindex >= targsize)
-		return (-1);
-	      target[tarindex] = (pos - Base64) << 2;
-	    }
-	  state = 1;
-	  break;
-	case 1:
-	  if (target)
-	    {
-	      if ((size_t) tarindex + 1 >= targsize)
-		return (-1);
-	      target[tarindex] |= (pos - Base64) >> 4;
-	      target[tarindex + 1] = ((pos - Base64) & 0x0f) << 4;
-	    }
-	  tarindex++;
-	  state = 2;
-	  break;
-	case 2:
-	  if (target)
-	    {
-	      if ((size_t) tarindex + 1 >= targsize)
-		return (-1);
-	      target[tarindex] |= (pos - Base64) >> 2;
-	      target[tarindex + 1] = ((pos - Base64) & 0x03) << 6;
-	    }
-	  tarindex++;
-	  state = 3;
-	  break;
-	case 3:
-	  if (target)
-	    {
-	      if ((size_t) tarindex >= targsize)
-		return (-1);
-	      target[tarindex] |= (pos - Base64);
-	    }
-	  tarindex++;
-	  state = 0;
-	  break;
-	default:
-	  abort ();
-	}
+    case GSASL_PIN:
+      {
+	Gsasl_client_callback_pin cb_pin
+	  = gsasl_client_callback_pin_get (sctx->ctx);
+	if (!cb_pin)
+	  break;
+	res = cb_pin (sctx, sctx->suggestedpin, buf, &buflen);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    case GSASL_REALM:
+      {
+	Gsasl_client_callback_realm cb_realm
+	  = gsasl_client_callback_realm_get (sctx->ctx);
+	if (!cb_realm)
+	  break;
+	res = cb_realm (sctx, buf, &buflen);
+	if (res != GSASL_OK)
+	  break;
+	buf[buflen] = '\0';
+	gsasl_property_set (sctx, prop, buf);
+	break;
+      }
+
+    default:
+      break;
     }
 
-  /*
-   * We are done decoding Base-64 chars.  Let's see if we ended
-   * on a byte boundary, and/or with erroneous trailing characters.
-   */
-
-  if (ch == Pad64)
-    {				/* We got a pad char. */
-      ch = *src++;		/* Skip it, get next. */
-      switch (state)
-	{
-	case 0:		/* Invalid = in first position */
-	case 1:		/* Invalid = in second position */
-	  return (-1);
-
-	case 2:		/* Valid, means one byte of info */
-	  /* Skip any number of spaces. */
-	  for ((void) NULL; ch != '\0'; ch = *src++)
-	    if (!isspace (ch))
-	      break;
-	  /* Make sure there is another trailing = sign. */
-	  if (ch != Pad64)
-	    return (-1);
-	  ch = *src++;		/* Skip the = */
-	  /* Fall through to "single trailing =" case. */
-	  /* FALLTHROUGH */
-
-	case 3:		/* Valid, means two bytes of info */
-	  /*
-	   * We know this char is an =.  Is there anything but
-	   * whitespace after it?
-	   */
-	  for ((void) NULL; ch != '\0'; ch = *src++)
-	    if (!isspace (ch))
-	      return (-1);
-
-	  /*
-	   * Now make sure for cases 2 and 3 that the "extra"
-	   * bits that slopped past the last full byte were
-	   * zeros.  If we don't check them, they become a
-	   * subliminal channel.
-	   */
-	  if (target && target[tarindex] != 0)
-	    return (-1);
-	}
-    }
-  else
+  return gsasl_property_fast (sctx, prop);
+}
+
+int
+_gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
+			  Gsasl_property prop)
+{
+  char buf[BUFSIZ];
+  size_t buflen = BUFSIZ - 1;
+  int res;
+
+  /* Call obsolete callbacks. */
+
+  switch (prop)
     {
-      /*
-       * We ended by seeing the end of the string.  Make sure we
-       * have no partial bytes lying around.
-       */
-      if (state != 0)
-	return (-1);
+    case GSASL_VALIDATE_ANONYMOUS:
+      {
+	Gsasl_server_callback_anonymous cb_anonymous;
+	if (!sctx->anonymous_token)
+	  break;
+	cb_anonymous = gsasl_server_callback_anonymous_get (sctx->ctx);
+	if (!cb_anonymous)
+	  break;
+	res = cb_anonymous (sctx, sctx->anonymous_token);
+	return res;
+	break;
+      }
+
+    case GSASL_VALIDATE_EXTERNAL:
+      {
+	Gsasl_server_callback_external cb_external
+	  = gsasl_server_callback_external_get (sctx->ctx);
+	if (!cb_external)
+	  break;
+	res = cb_external (sctx);
+	return res;
+	break;
+      }
+
+    case GSASL_VALIDATE_SECURID:
+      {
+	Gsasl_server_callback_securid cb_securid
+	  = gsasl_server_callback_securid_get (sctx->ctx);
+	if (!cb_securid)
+	  break;
+	res = cb_securid (sctx, sctx->authid, sctx->authzid, sctx->passcode,
+			  sctx->pin, buf, &buflen);
+	if (buflen > 0 && buflen < BUFSIZ - 1)
+	  {
+	    buf[buflen] = '\0';
+	    gsasl_property_set (sctx, GSASL_SUGGESTED_PIN, buf);
+	  }
+	return res;
+	break;
+      }
+
+    case GSASL_VALIDATE_GSSAPI:
+      {
+	Gsasl_server_callback_gssapi cb_gssapi
+	  = gsasl_server_callback_gssapi_get (sctx->ctx);
+	if (!cb_gssapi)
+	  break;
+	res = cb_gssapi (sctx, sctx->gssapi_display_name, sctx->authzid);
+	return res;
+	break;
+      }
+
+    case GSASL_VALIDATE_SIMPLE:
+      {
+	Gsasl_server_callback_validate cb_validate
+	  = gsasl_server_callback_validate_get (sctx->ctx);
+	if (!cb_validate)
+	  break;
+	res = cb_validate (sctx, sctx->authzid, sctx->authid, sctx->password);
+	return res;
+	break;
+      }
+
+    case GSASL_PASSWORD:
+      {
+	Gsasl_server_callback_retrieve cb_retrieve
+	  = gsasl_server_callback_retrieve_get (sctx->ctx);
+	if (!cb_retrieve)
+	  break;
+	res = cb_retrieve (sctx, sctx->authid, sctx->authzid,
+			   sctx->hostname, buf, &buflen);
+	if (res == GSASL_OK)
+	  gsasl_property_set_raw (sctx, GSASL_PASSWORD, buf, buflen);
+	/* FIXME else if (res == GSASL_TOO_SMALL_BUFFER)... */
+	return res;
+	break;
+      }
+
+    default:
+      break;
     }
 
-  return (tarindex);
+  return GSASL_NO_CALLBACK;
 }
