@@ -38,9 +38,7 @@
 #include <poll.h>		/* poll() */
 #include <unistd.h>
 #include <netinet/in.h>
-#ifdef HAVE_NETDB_H
-# include <netdb.h>
-#endif
+#include <netdb.h>
 #ifdef HAVE_PWD_H
 # include <pwd.h>		/* getpwnam */
 #endif
@@ -51,7 +49,6 @@
 #include "progname.h"
 #include "error.h"
 #include "getpass.h"
-#include "getaddrinfo.h"
 #include "readline.h"
 #include "quote.h"
 #include "version-etc.h"
